package com.jiuqi.sql;

package .tmp;

import com.jiuqi.sql.DBException;
import com.jiuqi.sql.DataSetEvent;
import com.jiuqi.sql.DataSetListener;
import com.jiuqi.sql.FieldInfo;
import com.jiuqi.sql.FieldInfoList;
import com.jiuqi.sql.InputDataSetMetaData;
import com.jiuqi.sql.InputRecordBuffer;
import java.util.Date;

public abstract class InputDataSet {
  private boolean active;
  
  private FieldInfoList fieldInfos;
  
  private InputRecordBuffer buffer;
  
  private DataSetListener listener = null;
  
  public void resetListener(DataSetListener listerner) {
    this.listener = listerner;
  }
  
  private FieldInfo findField(String fieldName) throws DBException {
    int index = this.fieldInfos.find(fieldName);
    if (index < 0)
      throw new DBException(String.valueOf(fieldName) + " Not Exists"); 
    FieldInfo result = this.fieldInfos.get(index);
    return result;
  }
  
  public abstract InputDataSetMetaData getMetaData() throws DBException;
  
  protected abstract boolean getNextRecord(InputRecordBuffer paramInputRecordBuffer) throws DBException;
  
  protected abstract void initFields(FieldInfoList paramFieldInfoList) throws DBException;
  
  protected abstract InputRecordBuffer internalOpen() throws DBException;
  
  protected abstract void internalClose() throws DBException;
  
  public InputDataSet() {
    this.fieldInfos = new FieldInfoList();
  }
  
  protected InputRecordBuffer recordBuffer() {
    return this.buffer;
  }
  
  public void open() throws DBException {
    if (!isActive()) {
      initFields(this.fieldInfos);
      this.buffer = internalOpen();
      this.fieldInfos.refreshFinder();
      this.active = true;
      if (this.listener != null)
        this.listener.afterEvent(new DataSetEvent(this, 1)); 
    } 
  }
  
  public void first() throws DBException {
    throw new DBException("尚未实现！");
  }
  
  public boolean isActive() {
    return this.active;
  }
  
  public boolean next() throws DBException {
    boolean result = false;
    while (true) {
      result = getNextRecord(this.buffer);
      if (result && this.buffer.isDeleted())
        continue; 
      break;
    } 
    return result;
  }
  
  public void close() throws DBException {
    if (isActive()) {
      internalClose();
      this.active = false;
      if (this.listener != null)
        this.listener.afterEvent(new DataSetEvent(this, 2)); 
    } 
  }
  
  public int fieldIndexOf(String fieldName) {
    int result = this.fieldInfos.find(fieldName);
    return (result < 0) ? result : (result + 1);
  }
  
  public boolean getBoolean(int index) throws DBException {
    return this.buffer.readBoolean(this.fieldInfos.get(index - 1));
  }
  
  public boolean getBoolean(String fieldName) throws DBException {
    return this.buffer.readBoolean(findField(fieldName));
  }
  
  public int getInt(int index) throws DBException {
    return this.buffer.readInt(this.fieldInfos.get(index - 1));
  }
  
  public int getInt(String fieldName) throws DBException {
    return this.buffer.readInt(findField(fieldName));
  }
  
  public long getLong(int index) throws DBException {
    return this.buffer.readLong(this.fieldInfos.get(index - 1));
  }
  
  public long getLong(String fieldName) throws DBException {
    return this.buffer.readLong(findField(fieldName));
  }
  
  public String getString(int index) throws DBException {
    return this.buffer.readString(this.fieldInfos.get(index - 1));
  }
  
  public String getString(String fieldName) throws DBException {
    return this.buffer.readString(findField(fieldName));
  }
  
  public double getDouble(int index) throws DBException {
    return this.buffer.readDouble(this.fieldInfos.get(index - 1));
  }
  
  public double getDouble(String fieldName) throws DBException {
    return this.buffer.readDouble(findField(fieldName));
  }
  
  public Date readDate(int index) throws DBException {
    return this.buffer.readDate(this.fieldInfos.get(index - 1));
  }
  
  public Date readDate(String fieldName) throws DBException {
    return this.buffer.readDate(findField(fieldName));
  }
  
  public Date getDate(int index) throws DBException {
    return this.buffer.readDate(this.fieldInfos.get(index - 1));
  }
  
  public Date getDate(String fieldName) throws DBException {
    return this.buffer.readDate(findField(fieldName));
  }
  
  private Object getObject(FieldInfo fieldInfo) throws DBException {
    long l;
    boolean bool;
    Date date;
    double d;
    int i;
    String value;
    switch (fieldInfo.type) {
      case 7:
      case 8:
        l = this.buffer.readLong(fieldInfo);
        return this.buffer.wasNull() ? null : new Long(l);
      case 9:
        return null;
      case 1:
        bool = this.buffer.readBoolean(fieldInfo);
        return this.buffer.wasNull() ? null : new Boolean(bool);
      case 2:
        date = this.buffer.readDate(fieldInfo);
        return this.buffer.wasNull() ? null : date;
      case 3:
        d = this.buffer.readDouble(fieldInfo);
        return this.buffer.wasNull() ? null : new Double(d);
      case 5:
        i = this.buffer.readInt(fieldInfo);
        return this.buffer.wasNull() ? null : new Integer(i);
      case 6:
        value = this.buffer.readString(fieldInfo);
        return this.buffer.wasNull() ? null : value;
    } 
    return null;
  }
  
  public final Object getObject(int index) throws DBException {
    return getObject(this.fieldInfos.get(index - 1));
  }
  
  public final Object getObject(String fieldName) throws DBException {
    return getObject(findField(fieldName));
  }
  
  public boolean wasNull() throws DBException {
    return this.buffer.wasNull();
  }
}
